@@ -98,6 +98,9 @@ class ContainerLauncher:
         max_spot_interruption_frequency = 10
         max_spot_instance_type_candidates = 3
         spot_allocation_strategy = 'capacity-optimized-prioritized'
+        iam_ecs_task_role_arn = None
+        iam_ecs_task_execution_role_arn = None
+        iam_ec2_role_arn = None
         if container_input.get('spot') is not None:
             if container_input['spot'].get('max_interruption_frequency') is not None:
                 max_spot_interruption_frequency = container_input['spot']['max_interruption_frequency']
@@ -109,11 +112,18 @@ class ContainerLauncher:
             if container_input['iam'].get('ecs_task') is not None:
                 if container_input['iam']['ecs_task'].get('policy_documents'):
                     iam_ecs_task_policy_documents = container_input['iam']['ecs_task']['policy_documents']
+                if container_input['iam']['ecs_task'].get('arn'):
+                    iam_ecs_task_role_arn = container_input['iam']['ecs_task']['arn']
             if container_input['iam'].get('ec2') is not None:
                 if container_input['iam']['ec2'].get('policy_documents'):
                     iam_ec2_policy_documents = container_input['iam']['ec2']['policy_documents']
                 if container_input['iam']['ec2'].get('managed_policy_arns'):
                     iam_ec2_managed_policy_arns = container_input['iam']['ec2']['managed_policy_arns']
+                if container_input['iam']['ec2'].get('arn'):
+                    iam_ec2_role_arn = container_input['iam']['ec2']['arn']
+            if container_input['iam'].get('ecs_task_execution') is not None:
+                if container_input['iam']['ecs_task_execution'].get('arn'):
+                    iam_ecs_task_execution_role_arn = container_input['iam']['ecs_task_execution']['arn']
         try:
             self.__options['name_suffix'] = container_input.get('name_suffix', str(uuid.uuid4()).replace('-', ''))
 
@@ -172,13 +182,21 @@ class ContainerLauncher:
             }).get_image_id({'instance_type': instance_type})
             subnet_ids = container_input['subnet_ids']
 
-            ec2_iam_role_arn = self.__create_ec2_iam_role(iam_ec2_managed_policy_arns, iam_ec2_policy_documents)
+            # IAM Roles
+            if iam_ec2_role_arn is not None:
+                ec2_iam_role_arn = iam_ec2_role_arn
+                self.__resources['ec2_iam_role_arn'] = iam_ec2_role_arn
+            else:
+                ec2_iam_role_arn = self.__create_ec2_iam_role(iam_ec2_managed_policy_arns, iam_ec2_policy_documents)
+            if iam_ecs_task_role_arn is None:
+                self.__create_ecs_task_iam_role(iam_ecs_task_policy_documents)
+            if iam_ecs_task_execution_role_arn is None:
+                self.__create_ecs_task_execution_iam_role()
 
-            self.__create_ecs_task_iam_role(iam_ecs_task_policy_documents)
-            self.__create_ecs_task_execution_iam_role()
             account_id = ec2_iam_role_arn.split(':')[4]
             self.__create_instance_profile()
-            self.__add_role_to_instance_profile()
+            ec2_role_name = ec2_iam_role_arn.split('/')[-1]
+            self.__add_role_to_instance_profile(ec2_role_name)
 
             self.__create_launch_template({
                 'ami': ami,
@@ -221,7 +239,8 @@ class ContainerLauncher:
                 commands=commands
             )
 
-            self.__create_ecs_task_definition(ecs_task_definition_data, instance_type)
+            self.__create_ecs_task_definition(ecs_task_definition_data, instance_type, iam_ecs_task_role_arn,
+                                              iam_ecs_task_execution_role_arn)
 
             self.__dynamodb.update_state(entry_id, self.__resources)
 
@@ -504,24 +523,33 @@ class ContainerLauncher:
                 )
                 self.logger.debug(
                     f"The EC2 IAM role {self.__resources['ec2_iam_role_name']} has been removed from the {self.__resources['instance_profile_name']} instance profile")
+            if self.__resources.get('ec2_iam_role_arn') and self.__resources.get('instance_profile_name'):
+                ec2_role_name = self.__resources['ec2_iam_role_arn'].split('/')[-1]
+                self.clients['iam'].remove_role_from_instance_profile(
+                    InstanceProfileName=self.__resources['instance_profile_name'],
+                    RoleName=ec2_role_name
+                )
+                self.logger.debug(
+                    f"The EC2 IAM role {ec2_role_name} has been removed from the {self.__resources['instance_profile_name']} instance profile")
         except Exception as e:
             self.logger.warning(f'An error occurred during the remove_role_from_instance_profile operation: {str(e)}')
         try:
-            for managed_policy_arn in self.__resources['ec2_iam_role_managed_policy_arns']:
-                self.clients['iam'].detach_role_policy(
-                    RoleName=self.__resources.get('ec2_iam_role_name'),
-                    PolicyArn=managed_policy_arn
-                )
-            ec2_iam_role_policy_document_names = self.__resources['ec2_iam_role_policy_document_names']
-            if self.__resources.get('ec2_iam_role_name') and ec2_iam_role_policy_document_names and len(
-                    ec2_iam_role_policy_document_names) > 0:
-                for ec2_iam_role_policy_document_name in ec2_iam_role_policy_document_names:
-                    self.clients['iam'].delete_role_policy(
-                        RoleName=self.__resources['ec2_iam_role_name'],
-                        PolicyName=ec2_iam_role_policy_document_name
+            if self.__resources.get('ec2_iam_role_name'):
+                for managed_policy_arn in self.__resources['ec2_iam_role_managed_policy_arns']:
+                    self.clients['iam'].detach_role_policy(
+                        RoleName=self.__resources.get('ec2_iam_role_name'),
+                        PolicyArn=managed_policy_arn
                     )
-                self.logger.debug(
-                    f"The IAM policies {str(self.__resources['ec2_iam_role_policy_document_names'])} has been deleted")
+                ec2_iam_role_policy_document_names = self.__resources['ec2_iam_role_policy_document_names']
+                if self.__resources.get('ec2_iam_role_name') and ec2_iam_role_policy_document_names and len(
+                        ec2_iam_role_policy_document_names) > 0:
+                    for ec2_iam_role_policy_document_name in ec2_iam_role_policy_document_names:
+                        self.clients['iam'].delete_role_policy(
+                            RoleName=self.__resources['ec2_iam_role_name'],
+                            PolicyName=ec2_iam_role_policy_document_name
+                        )
+                    self.logger.debug(
+                        f"The IAM policies {str(self.__resources['ec2_iam_role_policy_document_names'])} has been deleted")
         except Exception as e:
             self.logger.warning(f'An error occurred during the delete_role_policy operation: {str(e)}')
         try:
@@ -772,12 +800,13 @@ class ContainerLauncher:
 
         self.logger.debug(f'The ECS cluster {ecs_cluster_name} has been created')
 
-    def __add_role_to_instance_profile(self):
+    def __add_role_to_instance_profile(self, role_name=None):
+        role_name = role_name if role_name is not None else self.__resources['ec2_iam_role_name']
         self.logger.debug(
-            f"Adding the {self.__resources['ec2_iam_role_name']} EC2 IAM role to the {self.__resources['instance_profile_name']} instance profile")
+            f"Adding the {role_name} EC2 IAM role to the {self.__resources['instance_profile_name']} instance profile")
         self.clients['iam'].add_role_to_instance_profile(
             InstanceProfileName=self.__resources['instance_profile_name'],
-            RoleName=self.__resources['ec2_iam_role_name']
+            RoleName=role_name
         )
 
     def __get_azs_by_subnet_ids(self, subnet_ids):
@@ -937,7 +966,12 @@ class ContainerLauncher:
         self.__resources['asg_scale_in_alarm_name'] = alarm_name
         self.logger.debug(f'The cloudwatch alarm {alarm_name} has been created')
 
-    def __create_ecs_task_definition(self, task_def_opts, instance_type):
+    def __create_ecs_task_definition(self, task_def_opts, instance_type, task_role_arn=None,
+                                     task_execution_role_arn=None):
+        task_role_name = task_role_arn.split('/')[-1] if task_role_arn is not None else self.__resources[
+            'ecs_task_iam_role_name']
+        task_execution_role_name = task_execution_role_arn.split('/')[-1] if task_execution_role_arn is not None else \
+        self.__resources['ecs_task_execution_iam_role_name']
         name_suffix = self.__options['name_suffix']
         task_definition_name = f'{self.__name_prefix}-task-def-{name_suffix}'
 
@@ -952,8 +986,8 @@ class ContainerLauncher:
 
         response = self.clients['ecs'].register_task_definition(
             family=task_definition_name,
-            taskRoleArn=self.__resources['ecs_task_iam_role_name'],
-            executionRoleArn=self.__resources['ecs_task_execution_iam_role_name'],
+            taskRoleArn=task_role_name,
+            executionRoleArn=task_execution_role_name,
             networkMode='host',
             containerDefinitions=[
                 {
